/*  This file is part of FediBlock-backend.
 *  Copyright © 2020 tastytea <tastytea@tastytea.de>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, version 3.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "time.hpp"

#include <chrono>
#include <cstdint>
#include <ctime>
#include <iomanip>
#include <sstream>
#include <string>
#include <string_view>

namespace FediBlock::time
{

using std::get_time;
using std::strftime;
using std::string;
using std::string_view;
using std::stringstream;
using std::time_t;
using std::uint16_t;
using std::chrono::system_clock;

string to_string(const system_clock::time_point &timepoint,
                 const string_view format)
{
    constexpr uint16_t bufsize = 1024;
    time_t time = system_clock::to_time_t(timepoint);
    std::tm *tm{nullptr};
    tm = std::gmtime(&time);
    char buffer[bufsize];
    strftime(buffer, bufsize, format.data(), tm);

    return buffer;
}

string to_string(const string_view timestring, const string_view format)
{
    std::tm tm{};
    tm.tm_isdst = -1; // Detect daylight saving time.
    stringstream ss(timestring.data());
    ss >> get_time(&tm, "%Y-%m-%dT%T"); // Assume time is UTC (%F doesn't work).

    return to_string(system_clock::from_time_t(timegm(&tm)), format);
}

} // namespace FediBlock::time
